@@ -11,27 +11,21 @@ from scipy.optimize import fmin_tnc
 from matplotlib import pyplot as plt
 from ffta.simulation.cantilever import Cantilever
 from ffta.pixel_utils.load import cantilever_params
+from ffta.pixel import Pixel
 import warnings
 
-def poly2(t, a, b, c):
-    return a * t ** 2 + b * t + c
+from pycroscopy.processing.fft import get_noise_floor
+from pycroscopy.analysis.utils.be_sho import SHOfunc, SHOestimateGuess, SHOfit
+import pycroscopy as px
+from igor.binarywave import load as loadibw
+import pyUSID as usid
 
-def cost_func(resp_wfm, signal):
+class GKPixel(Pixel):
 
-    cost = lambda p: np.sum((poly2(resp_wfm, *p) - signal) ** 2)
-
-    pinit = [-1 * np.abs(np.max(signal) - np.min(signal)), 0, 0]
-
-    popt, _, _ = fmin_tnc(cost, pinit, approx_grad=True, disp=0,
-                          bounds=[(-10, 10),
-                                  (-10, 10),
-                                  (-10, 10)])
-
-    return popt
-
-class GKPixel:
-
-    def __init__(self, signal_array, params, TF_norm=[], periods = 2):
+    def __init__(self, signal_array, params, can_params={},
+                 fit=True, pycroscopy=False, method='hilbert', fit_form='product', 
+                 filter_amplitude=False, filter_frequency=False, 
+                 TF_norm=[], exc_wfm=[], periods = 2, phase_shift=0):
         '''
         Class for processing G-KPFM data
 
@@ -41,17 +35,16 @@ class GKPixel:
             Assigns that as the CPD
 
         Parameters:
-            signal_array : h5Py Dataset or USIDDataset
-                This currently only works on data that is one signal per pixel (i.e already averaged/flattened)
-            params : dict
-                Specifies parameters for analysis. Should include drive_freq, sampling_rate, total_time.
+            See Pixel of parameter defintions
+            
+            Additional parameters:
+                
             TF_norm : array, optional
                 Transfer function supplied in Shifted Fourier domain, normalized to desired Q
             periods: int
-                Number of periods to average over
-
-        ncycles : int
-            number of cantilever cycles to average over
+                Number of periods to average over for CPD calc
+            phase_shift : float
+                Amount to shift the phase of the deflection by (cable lag)
 
         Returns:
             CPD : array
@@ -61,38 +54,32 @@ class GKPixel:
             CPD_mean : float
                 Simple average of the CPD trace, useful for plotting
         '''
+        self.periods = periods
+        self.phase_shift = phase_shift
 
-        self.signal_array = signal_array
+        super().__init__(signal_array, params, can_params,
+                                      fit, False, method, fit_form, 
+                                      filter_amplitude, filter_frequency)
 
         # This functionality is for single lines
-        if len(signal_array.shape) > 1:
+        if len(self.signal_array.shape) > 1:
             warnings.warn('This function only works on 1D (single lines). Flattening..')
             self.signal_array.flatten()
 
-        for key, value in params.items():
-            setattr(self, key, value)
-
-        self.n_points = len(signal_array)
-        self.periods = periods
-        
-        self.pxl_time = self.n_points/self.sampling_rate   # how long each pixel is in time (8.192 ms)
-        self.time_per_osc = (1/self.drive_freq) # period of drive frequency 
-        self.pnts_per_period = self.sampling_rate * self.time_per_osc  # points in a cycle 
-        self.num_periods = int(self.pxl_time/self.time_per_osc)  # number of periods in each pixel
-        
-        self.num_CPD = int(np.floor(self.num_periods / self.periods))  # number of CPD samples, since each CPD takes some number of periods
-        self.pnts_per_CPD = int(np.floor(self.pnts_per_period * self.periods))  # points used to calculate CPD
-        self.remainder = int(self.n_points % self.pnts_per_CPD)
+        self.n_points = len(self.signal_array)
 
         self.t_ax = np.linspace(0, self.total_time, self.n_points) #time axis
         self.f_ax = np.linspace(-self.sampling_rate/2, self.sampling_rate/2, num=self.n_points)
         
-        self.excitation()
+        self.SIG = np.fft.fftshift(np.fft.fft(self.signal_array))
         
-        self.SIG = np.fft.fftshift(np.fft.fft(signal_array))
         self.TF_norm = []
         if any(TF_norm):
-            self.TF_norm = TF
+            self.TF_norm = TF_norm
+        
+        self.exc_wfm = exc_wfm
+        if not any(exc_wfm):
+            self.excitation()
         
         return
     
@@ -145,6 +132,112 @@ class GKPixel:
         self.sig_dc = sig_dc
         
         return
+    
+    def load_tf(self, tf_path, excitation_path, remove_dc = False):
+        '''
+        Process transfer function and broadband excitation from supplied file
+        This function does not check shape or length
+        '''
+        tf = loadibw(tf_path)['wave']['wData']
+        exc = loadibw(excitation_path)['wave']['wData']
+        self.tf = np.mean(tf, axis=1)
+        self.TF = np.fft.fftshift(np.fft.fft(self.tf))
+        
+        if remove_dc:
+            self.TF[int(len(tf)/2)] = 0
+
+        self.exc = np.mean(exc, axis=1)
+        self.EXC = np.fft.fftshift(np.fft.fft(self.exc))
+
+    def process_tf(self, resonances = 2, width = 20e3, exc_floor = 10, plot=False):
+        '''
+        Parameters
+        ----------
+        resonances : int, optional
+            Number of resonances to fit SHO to. The default is 2.
+        width : int, optional
+            Width of resonance peaks to fit against. The default is 20e3.
+        exc_floot : float, optional
+            Sets the floor for the transfer function, below that is ignored. 
+            The default is 10
+        plot : bool, optional
+            Displays fits. The default is False
+    
+        Returns
+        -------
+        None.
+
+        '''
+    
+        #      
+        center = int(len(self.f_ax)/2)
+        df = self.sampling_rate / self.n_points
+        # drive_bin = int(np.ceil(self.drive_freq / df)) 
+        
+        # Reconstruct from SHO
+        excited_bins = np.where(np.abs(self.EXC) > exc_floor)[0]
+        band_edges = tf_fit_mat(self.drive_freq, resonances, width)
+    
+        Q_guesses = [self.Q, 3*self.Q, 6*self.Q, 9*self.Q]
+        self.coef_mat = np.zeros((resonances,4))
+        
+        TF = np.zeros(len(self.TF))
+      
+        if plot:
+            fig = plt.figure(10, figsize=(8,int(4*resonances)), facecolor='white')  
+      
+        # Constructs effective SHO
+        for n, q_guess in enumerate(Q_guesses[:resonances]):
+            
+            # find the bins in frequency axis if above exc_floor threshold
+            bin_lo, bin_hi = np.ceil(band_edges[n] / df).astype(int) + center
+            _exc = np.intersect1d(excited_bins, np.arange(bin_lo, bin_hi))
+            band = self.f_ax[_exc]
+            response = self.TF[bin_lo:bin_hi]
+            
+            if not any(band):
+                _msg = 'Ignoring resonance '+str(n) + ' outside excitation band'
+                warnings.warn(_msg)
+                break
+            
+            # initial guesses    
+            a_guess = np.max(np.abs(response)) / q_guess
+            w_guess = band[int(len(band)/2)] 
+            phi_guess = -np.pi
+            coef_guess = [np.real(a_guess), w_guess, q_guess, phi_guess]
+
+            #SHO fit
+            # coef = SHOestimateGuess(response, band, 10)
+            coef = SHOfit(coef_guess, band, response)
+            self.coef_mat[n,:] = coef
+            response_guess = SHOfunc(coef_guess, band)
+            response_fit = SHOfunc(coef, band)         
+            response_full = SHOfunc(coef, self.f_ax)
+            TF = TF + response_full
+      
+            if plot:
+                
+                plt.subplot(resonances, 2, n+1)
+                plt.plot(band/1e6, np.abs(response),'b.-')
+                plt.plot(band/1e6, np.abs(response_guess), 'g')
+                plt.plot(band/1e6, np.abs(response_fit), 'r')
+                plt.xlabel('Frequency (MHz)')
+                plt.ylabel('Amplitude (nm)')
+                plt.tight_layout(pad=0.0, w_pad=0.0, h_pad=0.0)
+    
+                plt.subplot(resonances, 2, (n+1)+2)
+                plt.plot(band/1e6, np.angle(response),'.-')
+                plt.plot(band/1e6, np.angle(response_guess),'g')
+                plt.plot(band/1e6, np.angle(response_fit),'r')
+                plt.xlabel('Frequency (MHz)')
+                plt.ylabel('Phase (Rad)')
+                plt.tight_layout(pad=0.0, w_pad=0.0, h_pad=0.0)
+        
+        # Normalize to first resonance Q
+        self.TF_norm = self.coef_mat[0,2] * (TF - np.min(np.abs(TF))) /  \
+                                            (np.max(np.abs(TF)) - np.min(np.abs(TF)))
+        
+        return
 
     def generate_tf(self, can_params_dict={}, plot=False):
         """
@@ -241,7 +334,7 @@ class GKPixel:
 
         return
 
-    def force_out(self, plot=False):
+    def force_out(self, plot=False, noise_tolerance=1e-6):
         """
         Reconstructs force by dividing by transfer function
 
@@ -249,6 +342,8 @@ class GKPixel:
         ----------
         plot : bool, optional
             Generates plot of reconstructed force. The default is False.
+        noise_tolerance : float, optional
+            Use to determine noise_floor, The default is 1e-6
 
         Returns
         -------
@@ -257,10 +352,29 @@ class GKPixel:
         """
         if not any(self.TF_norm):
             raise AttributeError('Supply Transfer Function or use generate_tf()')
+
+        center = int(len(self.SIG)/2)
+        drive_bin = int(self.drive_freq / (self.sampling_rate / len(self.SIG))            )
+
+        SIG = self.SIG
+        
+        if self.phase_shift != 0:
+            SIG = self.SIG * np.exp(-1j * self.f_ax[drive_bin] * self.phase_shift)
             
-        self.FORCE = self.SIG / self.TF_norm
+        self.FORCE = np.zeros(len(SIG), dtype=complex)
+        
+        noise_limit = np.ceil(get_noise_floor(SIG, noise_tolerance))
+        
+        # Only save bins above the noise_limit
+        signal_pass = np.where(np.abs(SIG) > noise_limit)[0]
+        
+        if 2 * drive_bin + center not in signal_pass:
+            warnings.warn('Second resonance not in passband; increase noise_tolerance')
+        
+        self.FORCE[signal_pass] = SIG[signal_pass]
+        self.FORCE = self.FORCE / self.TF_norm
         self.force = np.real(np.fft.ifft(np.fft.ifftshift(self.FORCE)))
-    
+        
         if plot:
             start = int(0.5 * self.trigger * self.sampling_rate)
             stop = int(1.5 * self.trigger * self.sampling_rate)
@@ -272,8 +386,32 @@ class GKPixel:
     
         return 
     
-    def plot_response(self):
+    def noise_filter(self, bw=1e3, plot=True, noise_tolerance=1e-6):
+        """
+        Denoising filter for 50 kHz harmonics (electrical noise in the system)
         
+        bw : float, optional
+            Bandwidth for the notch filters
+        
+        """
+        nbf = px.processing.fft.NoiseBandFilter(len(self.force), self.sampling_rate, 
+                                        [2E3, 50E3, 100E3, 150E3, 200E3],
+                                        [4E3, bw, bw, bw, bw]) 
+       
+        filt_line, _, _ = px.processing.gmode_utils.test_filter(self.force,
+                                                                frequency_filters=nbf,
+                                                                noise_threshold=noise_tolerance,
+                                                                show_plots=plot)
+        self.force = np.real(filt_line)
+        self.FORCE = np.fft.fftshift(np.fft.fft(self.force))
+        
+        return
+    
+    def plot_response(self):
+        """
+        Plots the transfer function and calculated force
+
+        """
         plt.figure()
         plt.semilogy(self.f_ax, np.abs(self.SIG), 'b')
         plt.semilogy(self.f_ax, np.abs(self.TF_norm), 'g')
@@ -283,8 +421,44 @@ class GKPixel:
         plt.title('Frequency Response of the Data')
 
         return
+    
+    def _calc_cpd_params(self, periods=2, return_dict=False):
+        """
+        Calculates the parameters needed to calculate the CPD
 
-    def analyze(self, verbose=False, deg = 2):
+        Parameters
+        ----------
+        periods : int, optional
+            Number of cantilever cycles to average over. The default is 2.
+        return_dict : bool, optional
+            Dictionary of these parameters for debugging purposes
+        """
+        
+        self.periods = periods
+        
+        self.pxl_time = self.n_points/self.sampling_rate   # how long each pixel is in time (8.192 ms)
+        self.time_per_osc = (1/self.drive_freq) # period of drive frequency 
+        self.pnts_per_period = self.sampling_rate * self.time_per_osc  # points in a cycle 
+        self.num_periods = int(self.pxl_time/self.time_per_osc)  # number of periods in each pixel
+        
+        self.num_CPD = int(np.floor(self.num_periods / self.periods))  # number of CPD samples, since each CPD takes some number of periods
+        self.pnts_per_CPD = int(np.floor(self.pnts_per_period * self.periods))  # points used to calculate CPD
+        self.remainder = int(self.n_points % self.pnts_per_CPD)
+        
+        if return_dict:
+            _cpdd = {'pxl_time': self.pxl_time,
+                     'time_per_osc': self.time_per_osc,
+                     'pnts_per_period': self.pnts_per_period,
+                     'num_periods': self.num_periods,
+                     'num_CPD': self.num_CPD,
+                     'pnts_per_CPD': self.pnts_per_CPD,
+                     'remainder': self.remainder}
+            return _cpdd
+        
+        return 
+
+    def analyze_cpd(self, verbose=False, deg = 2, use_raw=False, periods=2, 
+                    overlap=False):
         """
         Extracts CPD and capacitance gradient from data.
         Parameters:
@@ -292,32 +466,75 @@ class GKPixel:
 
             deg: int
                 Degree of polynomial fit. Default is 2, which is a quadratic fit.
+                Unless there's a good reason, quadratic is correct to use
+            
+            use_raw : bool, optional
+                Uses the signal_array instead of the reconstructed force
+                
+            periods : int, optional
+                Numer of cantilever cycles to average over for CPD extraction
+                
+            overlap : bool, optional
+                If False, each CPD is from a separate part of the signal. 
+                If True, shifts signal by 1 pixel and recalculates
         """
 
-        #        tx = np.arange(0,self.total_time, self.total_time/len(self.signal_array))
-        #        tx_cycle = np.arange(0, self.total_time, self.ncycles * self.total_time/len(self.signal_array))
+        self._calc_cpd_params(periods)
 
-        num_CPD = self.num_CPD
         pnts = self.pnts_per_CPD
-
-        self.t_ax_wH = np.linspace(0, self.periods*self.time_per_osc*self.num_CPD, num_CPD) #time ax for CPD/capacitance
-        test_wH = np.zeros((num_CPD, deg+1))
+        step = pnts
         
-        for p in range(num_CPD):
+        if overlap: 
+            self.t_ax_wH = np.copy(self.t_ax)
+            step=1
+        
+        cpd_px = np.arange(0, self.n_points , step)
+        test_wH = np.zeros((len(cpd_px), deg+1))
+        
+        for n, p in enumerate(cpd_px):
+        
+            if use_raw:
 
-            resp_x = np.float32(self.signal_array[pnts*p:pnts*(p+1)])
+                resp_x = np.float32(self.signal_array[p:p+pnts])
+
+            else:
+
+                resp_x = np.float32(self.force[p:p+pnts])
+            
             resp_x -= np.mean(resp_x)
             
-            V_per_osc = self.exc_wfm[pnts*p:pnts*(p+1)]
-            #V_per_osc = excitation[:decimation] # testing single fit
+            V_per_osc = self.exc_wfm[p:p+pnts]
                     
             popt, _ = npPoly.polyfit(V_per_osc, resp_x, deg, full=True)
-            test_wH[p] = popt.flatten()
+            test_wH[n] = popt.flatten()
        
         self.test_wH = test_wH
         self.CPD =  -0.5 * test_wH[:,1]/test_wH[:,2]
         self.capacitance = test_wH[:,2]
-
+        
+        if any(np.argwhere(np.isnan(self.CPD))):
+            self.CPD[-1] = self.CPD[-2]
+            self.capacitance[-1] = self.capacitance[-2]
+    
+    def filter_cpd(self):
+        """
+        Filters the capacitance based on pixel parameter self.filter_bandwidth
+        (typical is 10 kHz, which is somewhat large)
+        """
+        center = int(len(self.CPD)/2)
+        df = self.sampling_rate / len(self.CPD)
+        bin_width = int(self.filter_bandwidth / df)
+        
+        _CPD = np.fft.fftshift(np.fft.fft(self.CPD))
+        _CPD[:center - bin_width] = 0
+        _CPD[center + bin_width:] = 0
+        self.CPD = np.real(np.fft.ifft(np.fft.ifftshift(_CPD)))
+        
+        _CPD = np.fft.fftshift(np.fft.fft(self.capacitance))
+        _CPD[:center - bin_width] = 0
+        _CPD[center + bin_width:] = 0
+        self.capacitance = np.real(np.fft.ifft(np.fft.ifftshift(_CPD)))
+        
         return 
 
     def min_phase(self, phases_to_test = [2.0708, 2.1208, 2.1708]):
@@ -338,52 +555,35 @@ class GKPixel:
         # have to iterate this cell many times to find the right phase
         phases_to_test = np.array(phases_to_test)
         
-        start = int(0.5 * self.trigger)
-        stop = int(1.5 * self.trigger)
+        start = int(0.5 * self.trigger * self.sampling_rate)
+        stop = int(1.5 * self.trigger * self.sampling_rate)
         
         mid = int(len(self.f_ax) / 2)
-        drive_bin = np.searchsorted(self.f_ax[mid:], self.drive_freq) + mid
+        drive_bin = int(self.drive_freq / (self.sampling_rate / len(self.SIG))) + mid
         
-        # that processes is very slow for large datasets
-        noise_floor = px.processing.fft.get_noise_floor(self.signal_array, 1e-6)[0]
-        Noiselimit = np.ceil(noise_floor)
+        numplots = len(phases_to_test)
+        fig, ax = plt.subplots(nrows=numplots, figsize=(6, int(4*numplots)), 
+                              facecolor='white')
         
-        fig, a = plt.subplots(nrows=3, figsize=(6, 14))
-        
-        for x, ph in zip(range(len(phases_to_test)), phases_to_test):
+        for x, ph in enumerate(phases_to_test):
            
-            SIG_shifted = self.SIG * np.exp(-1j * self.f_Z/self.f_Z[drive_bin] * ph)
+            #SIG_shifted = self.SIG * np.exp(-1j * self.f_ax/self.f_ax[drive_bin] * ph)
+            SIG_shifted = self.SIG * np.exp(-1j * self.f_ax[drive_bin] * ph)
             Gout_shifted = SIG_shifted / self.TF_norm
             gout_shifted = np.real(np.fft.ifft(np.fft.ifftshift(Gout_shifted)))
+            self.phase_shift = ph
+            self.force_out(plot=False)
+            usid.plot_utils.rainbow_plot(ax[x], self.exc_wfm, self.force)
+            ax[x].set_title('Phase='+str(ph))
+
+            # a[x].plot(self.exc_wfm[start:start+1000], gout_shifted[start:start+1000], 'b')
+            # a[x].plot(self.exc_wfm[stop:stop+1000], gout_shifted[stop:stop+1000], 'r')
+            # a[x].set_title('Phase='+str(ph))
+
+        print('Set self.phase_shift to match desired phase offset (radians)')
         
-            a[x].plot(self.exc_wfm[start:start+1000], gout_shifted[start:start+1000], 'b')
-            a[x].plot(self.exc_wfm[stop:stop+1000], gout_shifted[stop:stop+1000], 'r')
-
         return
 
-    def min_phase_old(self, signal):
-
-        fits = []
-        errors = []
-        xpts = np.arange(-2 * np.pi, 2 * np.pi, 0.1)
-
-        for i in xpts:
-            txl = np.linspace(0, self.total_time, self.n_points)
-            resp_wfm = np.sin(txl * 2 * np.pi * self.drive_freq + i)[:len(signal)]
-
-            mid = int(self.pts_per_cycle / 2)
-
-            # find fits for first half-cycle and second half-cycle
-            p1 = cost_func(resp_wfm[:mid], signal[:mid])
-            p2 = cost_func(resp_wfm[-mid:], signal[-mid:])
-
-            fit1 = -0.5 * p1[1] / p1[0]
-            fit2 = -0.5 * p2[1] / p2[0]
-
-            fits.append(np.abs(fit2 - fit1))
-
-        return xpts[np.argmin(fits)]
-
     def min_phase_fft(self, signal):
 
         fits = []
@@ -421,3 +621,30 @@ class GKPixel:
         ph = xpts[ph_test[np.argmin(fitsp)]]
 
         return ph
+
+# Support
+def poly2(t, a, b, c):
+    return a * t ** 2 + b * t + c
+
+def cost_func(resp_wfm, signal):
+
+    cost = lambda p: np.sum((poly2(resp_wfm, *p) - signal) ** 2)
+
+    pinit = [-1 * np.abs(np.max(signal) - np.min(signal)), 0, 0]
+
+    popt, _, _ = fmin_tnc(cost, pinit, approx_grad=True, disp=0,
+                          bounds=[(-10, 10),
+                                  (-10, 10),
+                                  (-10, 10)])
+
+    return popt
+
+def tf_fit_mat(drive_freq, resonances=2, width=20e3):
+
+    # Cantilever resonances, see Table 1 in doi:10.1016/j.surfrep.2005.08.003
+    eigen_factors = [1, 6.255, 17.521, 34.33]
+    
+    band_edge_mat = [np.array([drive_freq * e - width, drive_freq * e + width])  
+                     for e in eigen_factors[:resonances]]
+
+    return np.array(band_edge_mat)
\ No newline at end of file
