@@ -7,22 +7,17 @@ Created on Tue Feb 11 18:07:06 2020
 
 import pyUSID as usid
 import ffta
+from ffta.pixel import Pixel
 from ffta.pixel_utils import badpixels
 import os
 import numpy as np
-from ffta.hdf_utils import get_utils
+from ffta.load import get_utils
 from pyUSID.processing.comp_utils import parallel_compute
 from pyUSID.io.write_utils import Dimension
 import h5py
 
 from matplotlib import pyplot as plt
 
-'''
-To do:
-    Separate the instantaneous frequency and tFP/shift calculations
-'''
-
-
 class FFtrEFM(usid.Process):
     """
     Implements the pixel-by-pixel processing using ffta.pixel routines
@@ -45,7 +40,8 @@ class FFtrEFM(usid.Process):
     """
 
     def __init__(self, h5_main, parm_dict = {}, can_params = {}, 
-                 pixel_params ={}, if_only=False, override=False, **kwargs):
+                 pixel_params ={}, if_only=False, override=False, process_name = 'Fast_Free',
+                 **kwargs):
         """
         Parameters
         ----------
@@ -89,8 +85,9 @@ class FFtrEFM(usid.Process):
         """
 
         self.parm_dict = parm_dict
+        self.parm_dict['if_only'] = if_only
         if not any(parm_dict):
-            self.parm_dict = get_utils.get_params(h5_main)
+            self.parm_dict = usid.hdf_utils.get_attributes(h5_main)
             self.parm_dict.update({'if_only': if_only})
         
         for key, val in parm_dict.items():
@@ -107,8 +104,28 @@ class FFtrEFM(usid.Process):
         self.pixel_params = pixel_params    
         self.override = override
         
-        super(FFtrEFM, self).__init__(h5_main, 'Fast_Free', parms_dict=self.parm_dict, **kwargs)
+        super(FFtrEFM, self).__init__(h5_main, process_name, parms_dict=self.parm_dict, **kwargs)
         
+        # For accidental passing ancillary datasets from Pycroscopy, this will fail
+        # when pickling
+        if hasattr(self, 'Position_Indices'):
+            del self.Position_Indices
+        if hasattr(self, 'Position_Values'):
+            del self.Position_Values
+        if hasattr(self, 'Spectroscopic_Indices'):
+            del self.Spectroscopic_Indices
+        if hasattr(self, 'Spectroscopic_Values'):
+            del self.Spectroscopic_Values
+
+        if 'Position_Indices' in self.parm_dict:
+            del self.parm_dict['Position_Indices']
+        if 'Position_Values' in self.parm_dict:
+            del self.parm_dict['Position_Values']
+        if 'Spectroscopic_Indices' in self.parm_dict:
+            del self.parm_dict['Spectroscopic_Indices']
+        if 'Spectroscopic_Values' in self.parm_dict:
+            del self.parm_dict['Spectroscopic_Values']
+
         return
 
     def update_parm(self, **kwargs):
@@ -151,7 +168,7 @@ class FFtrEFM(usid.Process):
         # as an array, not an ffta.Pixel
         defl = get_utils.get_pixel(self.h5_main, pixel_ind, array_form=True)
 
-        pix = ffta.pixel.Pixel(defl, self.parm_dict, **self.pixel_params)
+        pix = Pixel(defl, self.parm_dict, **self.pixel_params)
 
         tfp, shift, inst_freq = pix.analyze()
         pix.plot()
@@ -312,6 +329,7 @@ class FFtrEFM(usid.Process):
         """
         
         if not self.override:
+            
             self.h5_results_grp = usid.hdf_utils.find_dataset(self.h5_main.parent, 'Inst_Freq')[index].parent
             self.h5_new_spec_vals = self.h5_results_grp['Spectroscopic_Values']
             self.h5_tfp = self.h5_results_grp['tfp']
@@ -349,7 +367,7 @@ class FFtrEFM(usid.Process):
         parm_dict = args[0]
         pixel_params = args[1]
 
-        pix = ffta.pixel.Pixel(defl, parm_dict, **pixel_params)
+        pix = Pixel(defl, parm_dict, **pixel_params)
 
         if parm_dict['if_only']:
             inst_freq, _, _ = pix.generate_inst_freq()
