@@ -17,6 +17,7 @@ from ffta.pixel_utils import noise
 from ffta.pixel_utils import parab
 from ffta.pixel_utils import fitting
 from ffta.pixel_utils import dwavelet
+from ffta.pixel_utils import tfp_calc
 
 from matplotlib import pyplot as plt
 
@@ -75,10 +76,11 @@ class Pixel:
         One of 
             hilbert: Hilbert transform method (default)
             wavelet: Morlet CWT approach
-            emd: Hilbert-Huang decomposition
-            fft: sliding FFT approach
-    filter_amp : bool, optional
+            stft: short time Fourier transform (sliding FFT)
+    filter_amplitude : bool, optional
         The Hilbert Transform amplitude can sometimes have drive frequency artifact.
+    filter_frequency : bool, optional
+        Filters the instantaneous frequency to remove noise peaks
 
     Attributes
     ----------
@@ -144,69 +146,73 @@ class Pixel:
     >>>
     >>> p = pixel.Pixel(signal_array, params)
     >>> tfp, shift, inst_freq = p.analyze()
+    >>>
+    >>> p.plot()
 
     """
 
     def __init__(self, signal_array, params, can_params={},
-                 fit=True, pycroscopy=False, 
-                 method='hilbert', fit_form='product', filter_amplitude=False):
+                 fit=True, pycroscopy=False,
+                 method='hilbert', fit_form='product', filter_amplitude=False,
+                 filter_frequency=False):
 
         # Create parameter attributes for optional parameters.
         # These defaults are overwritten by values in 'params'
-        
+
         # FIR (Hilbert) filtering parameters
         self.n_taps = 1499
         self.filter_bandwidth = 5000
-        
+        self.filter_frequency = filter_frequency
+
         # Wavelet parameters
         self.wavelet_analysis = False
-        self.wavelet = 'cmor1-1' # default complex Morlet wavelet
+        self.wavelet = 'cmor1-1'  # default complex Morlet wavelet
         self.scales = np.arange(100, 2, -1)
-        self.wavelet_params = {} # currently just optimize flag is supported
-        
+        self.wavelet_params = {}  # currently just optimize flag is supported
+
         # Short Time Fourier Transform
         self.fft_analysis = False
         self.fft_cycles = 2
-        self.fft_params = {} # for sliding FFT
-        
+        self.fft_params = {}  # for STFT
+
         self.recombination = False
         self.phase_fitting = False
         self.check_drive = True
-        
+
         # Assign the fit parameter.
         self.fit = fit
         self.fit_form = fit_form
         self.method = method
         self.filter_amplitude = filter_amplitude
-        
+
         # Default Cantilever parameters, plugging in some reasonable defaults
-        self.AMPINVOLS = 122e-9 
+        self.AMPINVOLS = 122e-9
         self.SpringConstant = 23.2
-        self.k = self.SpringConstant 
+        self.k = self.SpringConstant
         self.DriveAmplitude = 1.7e-9
         self.Mass = 4.55e-12
         self.Beta = 3114
         self.Q = 360
-        
+
         # Read parameter attributes from parameters dictionary.
         for key, value in params.items():
-            
             setattr(self, key, value)
-        
+
         for key, value in can_params.items():
-            
             setattr(self, key, float(value))
-                
+
+        if self.filter_frequency:
+            self.bandpass_filter = 0  # turns off FIR
+
         # Assign values from inputs.
         self.signal_array = signal_array
-        self.signal_orig = None # used in amplitude calc to undo any Windowing beforehand
+        self.signal_orig = None  # used in amplitude calc to undo any Windowing beforehand
         if pycroscopy:
             self.signal_array = signal_array.T
         self.tidx = int(self.trigger * self.sampling_rate)
 
         # Set dimensions correctly
         # Three cases: 1) 2D (has many averages) 2) 1D (but set as 1xN) and 3) True 1D
-
         if len(signal_array.shape) == 2 and 1 not in signal_array.shape:
 
             self.n_points, self.n_signals = self.signal_array.shape
@@ -230,8 +236,19 @@ class Pixel:
         self.tfp = None
         self.shift = None
         self.cwt_matrix = None
-        
-        self.verbose = False # for console feedback
+
+        self.verbose = False  # for console feedback
+
+        # For accidental passing ancillary datasets from Pycroscopy, this will fail
+        # when pickling
+        if hasattr(self, 'Position_Indices'):
+            del self.Position_Indices
+        if hasattr(self, 'Position_Values'):
+            del self.Position_Values
+        if hasattr(self, 'Spectroscopic_Indices'):
+            del self.Spectroscopic_Indices
+        if hasattr(self, 'Spectroscopic_Values'):
+            del self.Spectroscopic_Values
 
         return
 
@@ -309,6 +326,8 @@ class Pixel:
         rate = self.sampling_rate
         lpf = self.drive_freq * 0.1
         self.signal, _, _ = dwavelet.dwt_denoise(self.signal, lpf, rate / 2, rate)
+        
+        return
 
     def fir_filter(self):
         """Filters signal with a FIR bandpass filter."""
@@ -360,6 +379,83 @@ class Pixel:
 
         return
 
+    def amplitude_filter(self):
+        '''
+        Filters the drive signal out of the amplitude response
+        '''
+        AMP = np.fft.fftshift(np.fft.fft(self.amplitude))
+
+        DRIVE = self.drive_freq / (self.sampling_rate / self.n_points)  # drive location in frequency space
+        center = int(len(AMP) / 2)
+
+        # crude boxcar
+        AMP[:center - int(DRIVE / 2) + 1] = 0
+        AMP[center + int(DRIVE / 2) - 1:] = 0
+
+        self.amplitude = np.abs(np.fft.ifft(np.fft.ifftshift(AMP)))
+
+        return
+
+    def frequency_filter(self):
+        '''
+        Filters the instantaneous frequency around DC peak to remove noise
+        Uses self.filter_bandwidth for the frequency filter
+        '''
+        FREQ = np.fft.fftshift(np.fft.fft(self.inst_freq))
+
+        center = int(len(FREQ) / 2)
+
+        df = self.sampling_rate / self.n_points
+        drive_bin = int(np.ceil(self.drive_freq / df))
+        bin_width = int(self.filter_bandwidth / df)
+
+        if bin_width > drive_bin:
+            print('width exceeds first resonance')
+            bin_width = drive_bin - 1
+
+        FREQ[:center - bin_width] = 0
+        FREQ[center + bin_width:] = 0
+
+        self.inst_freq = np.real(np.fft.ifft(np.fft.ifftshift(FREQ)))
+
+        return
+
+    def frequency_harmonic_filter(self, width=5):
+        '''
+        Filters the instantaneous frequency to remove noise
+        Defaults to DC and then every multiple harmonic up to sampling
+        
+        width : int, optional
+            Size of the boxcar around the various peaks
+        '''
+        FREQ = np.fft.fftshift(np.fft.fft(self.inst_freq))
+
+        center = int(len(FREQ) / 2)
+
+        # Find drive_bin
+        df = self.sampling_rate / self.n_points
+        drive_bin = int(np.ceil(self.drive_freq / df))
+        bins = np.arange(len(FREQ) / 2)[::drive_bin]
+        bins = np.append(center - bins, center + bins)
+
+        FREQ_filt = np.zeros(len(FREQ), dtype='complex128')
+        for b in bins:
+            FREQ_filt[int(b) - width:int(b) + width] = FREQ[int(b) - width:int(b) + width]
+
+        self.inst_freq = np.real(np.fft.ifft(np.fft.ifftshift(FREQ)))
+
+        return
+
+    def hilbert(self):
+        """Analytical signal and calculate phase/frequency via Hilbert transform"""
+
+        self.hilbert_transform()
+        self.calculate_amplitude()
+        self.calculate_phase()
+        self.calculate_inst_freq()
+
+        return
+
     def hilbert_transform(self):
         """Gets the analytical signal doing a Hilbert transform."""
 
@@ -375,44 +471,28 @@ class Pixel:
             signal_orig = self.signal_array.mean(axis=1)
         else:
             signal_orig = self.signal_array
-            
+
         self.amplitude = np.abs(sps.hilbert(signal_orig))
-        
-        self.amplitude *= self.AMPINVOLS
+
+        if not np.isnan(self.AMPINVOLS):
+            self.amplitude *= self.AMPINVOLS
 
         return
-    
+
     def calculate_power_dissipation(self):
         """Calculates the power dissipation using amplitude, phase, and frequency
         and the Cleveland eqn (see DOI:10.1063/1.121434)"""
-        
-        phase = self.phase# + np.pi/2 #offsets the phase to be pi/2 at resonance
-        
+
+        phase = self.phase  # + np.pi/2 #offsets the phase to be pi/2 at resonance
+
         # check for incorrect values (some off by 1e9 in our code)
-       
-        A = self.k/self.Q * self.amplitude**2 * (self.inst_freq + self.drive_freq)
+
+        A = self.k / self.Q * self.amplitude ** 2 * (self.inst_freq + self.drive_freq)
         B = self.Q * self.DriveAmplitude * np.sin(phase) / self.amplitude
-        C = self.inst_freq /self.drive_freq
-        
+        C = self.inst_freq / self.drive_freq
+
         self.power_dissipated = A * (B - C)
-        
-        return 
-    
-    def amplitude_filter(self):
-        '''
-        Filters the drive signal out of the amplitude response
-        '''
-        AMP = np.fft.fftshift(np.fft.fft(self.amplitude)) 
-        
-        DRIVE = self.drive_freq/(self.sampling_rate/self.n_points) # drive location in frequency space
-        center = int(len(AMP)/2)
-        
-        # crude boxcar
-        AMP[:center-int(DRIVE/2)+1] = 0
-        AMP[center+int(DRIVE/2)-1:] = 0
-        
-        self.amplitude = np.abs( np.fft.ifft(np.fft.ifftshift(AMP)) )
-        
+
         return
 
     def calculate_phase(self, correct_slope=True):
@@ -437,9 +517,9 @@ class Pixel:
             # Remove the fit from phase.
             self.phase -= (xfit[0] * np.arange(self.n_points)) + xfit[1]
 
-        self.phase = -self.phase #need to correct for negative in DDHO solution
+        self.phase = -self.phase  # need to correct for negative in DDHO solution
 
-        self.phase += np.pi/2 # corrects to be at resonance pre-trigger
+        self.phase += np.pi / 2  # corrects to be at resonance pre-trigger
 
         return
 
@@ -451,10 +531,10 @@ class Pixel:
 
         # Do a Savitzky-Golay smoothing derivative
         # using 5 point 1st order polynomial.
-        
-        #-self.phase to correct for sign in DDHO solution
+
+        # -self.phase to correct for sign in DDHO solution
         self.inst_freq_raw = sps.savgol_filter(-self.phase, 5, 1, deriv=1,
-                                           delta=dtime)
+                                               delta=dtime)
 
         # Bring trigger to zero.
         self.tidx = int(self.tidx)
@@ -462,177 +542,39 @@ class Pixel:
 
         return
 
-    def find_minimum(self):
-        """Finds when the minimum of instantaneous frequency happens."""
-
-        # Cut the signal into region of interest.
+    def find_tfp(self):
+        """Calculate tfp and shift based self.fit_form and self.fit selection"""
         ridx = int(self.roi * self.sampling_rate)
         cut = self.inst_freq[self.tidx:(self.tidx + ridx)]
-
-        # Define a spline to be used in finding minimum.
-        x = np.arange(ridx)
-        y = cut
-
-        _spline_sz = 2 * self.sampling_rate / self.drive_freq
-        func = spi.UnivariateSpline(x, y, k=4, ext=3, s=_spline_sz)
-        
-        # Find the minimum of the spline using TNC method.
-        res = spo.minimize(func, cut.argmin(),
-                           method='TNC', bounds=((0, ridx),))
-        idx = res.x[0]
-
+        cut -= self.inst_freq[self.tidx]
         self.cut = cut
-        self.best_fit = func(np.arange(ridx))
-
-        # Do index to time conversion and find shift.
-        self.tfp = idx / self.sampling_rate
-        self.shift = func(0) - func(idx)
-
-        return
-
-    def fit_freq_product(self):
-        """Fits the frequency shift to an approximate functional form using
-        an analytical fit with bounded values."""
-
-        # Calculate the region of interest and if filtered move the fit index.
-        ridx = int(self.roi * self.sampling_rate)
-        fidx = self.tidx
-
-        # Make sure cut starts from 0 and never goes over.
-        cut = self.inst_freq[fidx:(fidx + ridx)] - self.inst_freq[fidx]
         t = np.arange(cut.shape[0]) / self.sampling_rate
 
-        # Fit the cut to the model.
-        popt = fitting.fit_bounded_product(self.Q, self.drive_freq, t, cut)
+        if not self.fit:
 
-        A, tau1, tau2 = popt
+            tfp_calc.find_minimum(self, cut)
 
-        # Analytical minimum of the fit.
-        # self.tfp = tau2 * np.log((tau1 + tau2) / tau2)
-        # self.shift = -A * np.exp(-self.tfp / tau1) * np.expm1(-self.tfp / tau2)
+        elif self.fit_form == 'sum':
 
-        # For diagnostic purposes.
-        self.cut = cut
-        self.popt = popt
-        self.best_fit = -A * (np.exp(-t / tau1) - 1) * np.exp(-t / tau2)
+            tfp_calc.fit_freq_sum(self, cut, t)
 
-        self.tfp = np.argmin(self.best_fit) / self.sampling_rate
-        self.shift = np.min(self.best_fit)
+        elif self.fit_form == 'exp':
 
-        self.rms = np.sqrt(np.mean(np.square(self.best_fit - cut)))
+            tfp_calc.fit_freq_exp(self, cut, t)
 
-        return
+        elif self.fit_form == 'ringdown':
 
-    def fit_freq_sum(self):
-        """Fits the frequency shift to an approximate functional form using
-        an analytical fit with bounded values."""
+            cut = self.amplitude[self.tidx:(self.tidx + ridx)]
+            tfp_calc.fit_ringdown(self, ridx, cut, t)
 
-        # Calculate the region of interest and if filtered move the fit index.
-        ridx = int(self.roi * self.sampling_rate)
-        fidx = self.tidx
+        elif self.fit_form == 'product':
 
-        # Make sure cut starts from 0 and never goes over.
-        cut = self.inst_freq[fidx:(fidx + ridx)] - self.inst_freq[fidx]
+            tfp_calc.fit_freq_product(self, cut, t)
 
-        t = np.arange(cut.shape[0]) / self.sampling_rate
+        elif self.fit_form == 'phase':
 
-        # Fit the cut to the model.
-        popt = fitting.fit_bounded_sum(self.Q, self.drive_freq, t, cut)
-        A1, A2, tau1, tau2 = popt
-
-        # For diagnostic purposes.
-        self.cut = cut
-        self.popt = popt
-        self.best_fit = A1 * (np.exp(-t / tau1) - 1) - A2 * np.exp(-t / tau2)
-
-        self.tfp = np.argmin(self.best_fit) / self.sampling_rate
-        self.shift = np.min(self.best_fit)
-
-        return
-
-    def fit_freq_exp(self):
-        """Fits the frequency shift to a single exponential in the case where
-        there is no return to 0 Hz offset (if drive is cut)."""
-
-        ridx = int(self.roi * self.sampling_rate)
-        fidx = self.tidx
-
-        # Make sure cut starts from 0 and never goes over.
-        cut = self.inst_freq[fidx:(fidx + ridx)] - self.inst_freq[fidx]
-
-        t = np.arange(cut.shape[0]) / self.sampling_rate
-
-        # Fit the cut to the model.
-        popt = fitting.fit_bounded_exp(t, cut)
-
-        # For diagnostics
-        A, y0, tau = popt
-        self.cut = cut
-        self.popt = popt
-        self.best_fit = A * (np.exp(-t / tau)) + y0
-
-        self.shift = A
-        self.tfp = tau
-
-        return
-    
-    def fit_ringdown(self):
-        """Fits the amplitude to determine Q from single exponential fit."""
-
-        ridx = int(self.roi * self.sampling_rate)
-        fidx = int(self.tidx)
-
-        cut = self.amplitude[fidx:(fidx + ridx)]
-
-        t = np.arange(cut.shape[0]) / self.sampling_rate
-
-        # Fit the cut to the model.
-        popt = fitting.fit_ringdown_exp(t, cut*1e9)
-        popt[0] *= 1e-9
-        popt[1] *= 1e-9
-
-        # For diagnostics
-        A, y0, tau = popt
-        self.cut = cut
-        self.popt = popt
-        self.best_fit = A * (np.exp(-t / tau)) + y0
-
-        self.shift = A
-        self.tfp = np.pi * self.drive_freq * tau # same as ringdown_Q to help with pycroscopy bugs that call tfp
-        self.ringdown_Q = np.pi * self.drive_freq * tau
-
-        return
-
-    def fit_phase(self):
-        """Fits the phase to an approximate functional form using an
-        analytical fit with bounded values."""
-
-        # Calculate the region of interest and if filtered move the fit index.
-        ridx = int(self.roi * self.sampling_rate)
-
-        fidx = self.tidx
-
-        # Make sure cut starts from 0 and never goes over.
-        # -1 on cut is because of sign error in generating phase
-        cut = -1 * (self.phase[fidx:(fidx + ridx)] - self.phase[fidx])
-        t = np.arange(cut.shape[0]) / self.sampling_rate
-
-        # Fit the cut to the model.
-        popt = fitting.fit_bounded_phase(self.Q, self.drive_freq, t, cut)
-
-        A, tau1, tau2 = popt
-
-        # Analytical minimum of the fit.
-        self.tfp = tau2 * np.log((tau1 + tau2) / tau2)
-        self.shift = A * np.exp(-self.tfp / tau1) * np.expm1(-self.tfp / tau2)
-
-        # For diagnostic purposes.
-        postfactor = (tau2 / (tau1 + tau2)) * np.exp(-t / tau2) - 1
-
-        self.cut = cut
-        self.popt = popt
-        self.best_fit = -A * np.exp(-t / tau1) * np.expm1(-t / tau2)
-        self.best_phase = A * tau1 * np.exp(-t / tau1) * postfactor + A * tau1 * (1 - tau2 / (tau1 + tau2))
+            cut = -1 * (self.phase[self.tidx:(self.tidx + ridx)] - self.phase[self.tidx])
+            tfp_calc.fit_phase(self, cut, t)
 
         return
 
@@ -653,7 +595,7 @@ class Pixel:
 
             self.phase = np.pad(self.phase, (d_trig, 0), 'edge')
             self.phase = self.phase[:self._n_points_orig]
-            
+
             self.amplitude = np.pad(self.amplitude, (d_trig, 0), 'edge')
             self.amplitude = self.amplitude[:self._n_points_orig]
 
@@ -674,7 +616,7 @@ class Pixel:
 
         return
 
-    def calculate_cwt(self, f_center = None, verbose=False, optimize = False, fit=False):
+    def calculate_cwt(self, f_center=None, verbose=False, optimize=False, fit=False):
         '''
         Calculate instantaneous frequency using continuous wavelet transfer
         
@@ -683,19 +625,19 @@ class Pixel:
         Optimize : bool, optionals
             Currently placeholder for iteratively determining wavelet scales
         '''
-        
-        #wavlist = pywt.wavelist(kind='continuous')
+
+        # wavlist = pywt.wavelist(kind='continuous')
         # w0, wavelet_increment, cwt_scale = self.__get_cwt__()
 
         # determine if scales will capture the relevant frequency
         if not f_center:
             f_center = self.drive_freq
-            
-        dt = 1/self.sampling_rate
+
+        dt = 1 / self.sampling_rate
         sc = pywt.scale2frequency(self.wavelet, self.scales) / dt
         if self.verbose:
-            print('Wavelet scale from', np.min(sc),'to', np.max(sc))
-            
+            print('Wavelet scale from', np.min(sc), 'to', np.max(sc))
+
         if f_center < np.min(sc) or f_center > np.max(sc):
             raise ValueError('Choose a scale that captures frequency of interest')
 
@@ -705,39 +647,41 @@ class Pixel:
             hi = int(1.2 * drive_bin)
             lo = int(0.8 * drive_bin)
             self.scales = np.arange(hi, lo, -0.1)
-        
-        spectrogram, freq = pywt.cwt(self.signal, self.scales,self.wavelet, sampling_period=dt)
-        
+
+        spectrogram, freq = pywt.cwt(self.signal, self.scales, self.wavelet, sampling_period=dt)
+
         if not fit:
-        
-            inst_freq, amplitude,_ = parab.ridge_finder(np.abs(spectrogram), np.arange(len(freq)))
-        
+
+            inst_freq, amplitude, _ = parab.ridge_finder(np.abs(spectrogram), np.arange(len(freq)))
+
         # slow serial curve fitting
         else:
+
             inst_freq = np.zeros(self.n_points)
             amplitude = np.zeros(self.n_points)
+
             for c in range(spectrogram.shape[1]):
-                SIG = spectrogram[:,c]
+
+                SIG = spectrogram[:, c]
                 if fit:
                     pk = np.argmax(np.abs(SIG))
-                    popt = np.polyfit(np.arange(20), 
+                    popt = np.polyfit(np.arange(20),
                                       np.abs(SIG[pk - 10:pk + 10]), 2)
                     inst_freq[c] = -0.5 * popt[1] / popt[0]
                     amplitude[c] = np.abs(SIG)[pk]
-        
 
         # rescale to correct frequency 
         inst_freq = pywt.scale2frequency(self.wavelet, inst_freq + self.scales[0]) / dt
-        
+
         phase = spg.cumtrapz(inst_freq)
         phase = np.append(phase, phase[-1])
-        tidx = int(self.tidx * len(inst_freq)/self.n_points)
+        tidx = int(self.tidx * len(inst_freq) / self.n_points)
 
         self.amplitude = amplitude
         self.inst_freq_raw = inst_freq
-        self.inst_freq = inst_freq - inst_freq[tidx]
+        self.inst_freq = -1 * (inst_freq - inst_freq[tidx])  # -1 due to way scales are ordered
         self.spectrogram = np.abs(spectrogram)
-        self.wavelet_freq = freq # the wavelet frequencies
+        self.wavelet_freq = freq  # the wavelet frequencies
 
         # subtract the w*t line (drive frequency line) from phase
         start = int(0.3 * tidx)
@@ -745,27 +689,17 @@ class Pixel:
         xfit = np.polyfit(np.arange(start, end), phase[start:end], 1)
         phase -= (xfit[0] * np.arange(len(inst_freq))) + xfit[1]
 
-        self.phase = phase 
-        
+        self.phase = phase
+
         return
 
     def calculate_stft(self, time_res=20e-6, fit=False, nfft=200):
         '''
         Sliding FFT approach
-        -Take self.fft_cycles number of cycles of data
-        -multiply by a window function
-        -Take FFT
-        -Find frequency corresponding to the peak
-        -set the data to that frequency
-        -slide by one pixel and repeat
         
         time_res : float, optional
             What timescale to evaluate each FFT over
-            
-        decimate : bool, optional
-            Whether to shift window by time_res or by single point
-            no decimate is quite slow but better visually
-        
+                  
         fit : bool, optional
             Fits a parabola to the frequency peak to get the actual frequency
             Otherwise defaults to parabolic interpolation (see parab.fit)
@@ -776,40 +710,54 @@ class Pixel:
         '''
 
         pts_per_ncycle = int(time_res * self.sampling_rate)
-            
-        #drivebin = int(self.drive_freq / (self.sampling_rate / nfft ))
+
+        if nfft < pts_per_ncycle:
+            print('Error with nfft setting')
+            nfft = pts_per_ncycle
+
+        # drivebin = int(self.drive_freq / (self.sampling_rate / nfft ))
         freq, times, spectrogram = sps.spectrogram(self.signal,
                                                    self.sampling_rate,
-                                                   nperseg = pts_per_ncycle,
-                                                   noverlap = pts_per_ncycle-1,
-                                                   nfft = nfft,
+                                                   nperseg=pts_per_ncycle,
+                                                   noverlap=pts_per_ncycle - 1,
+                                                   nfft=nfft,
                                                    window=self.window,
                                                    mode='magnitude')
-            
+
         # Parabolic ridge finder
         if not fit:
-            inst_freq, amplitude,_ = parab.ridge_finder(spectrogram, freq)
-        
+            inst_freq, amplitude, _ = parab.ridge_finder(spectrogram, freq)
+
         # slow serial curve fitting
         else:
-            
+
             for c in range(spectrogram.shape[1]):
-                SIG = spectrogram[:,c]
+
+                SIG = spectrogram[:, c]
                 if fit:
                     pk = np.argmax(np.abs(SIG))
                     popt = np.polyfit(freq[pk - 2:pk + 2], np.abs(SIG[pk - 2:pk + 2]), 2)
                     inst_freq[c] = -0.5 * popt[1] / popt[0]
                     amplitude[c] = np.abs(SIG)[pk]
-    
+
+        # Correctly pad the signals
+        _pts = self.n_points - len(inst_freq)
+        _pre = int(np.floor(_pts/2))
+        _post = int(np.ceil(_pts/2)) 
+        
+        inst_freq = np.pad(inst_freq, (_pre, _post))
+        amplitude = np.pad(amplitude, (_pre, _post))
+
         phase = spg.cumtrapz(inst_freq)
         phase = np.append(phase, phase[-1])
-        tidx = int(self.tidx * len(inst_freq)/self.n_points)
+        tidx = int(self.tidx * len(inst_freq) / self.n_points)
 
         self.amplitude = amplitude
         self.inst_freq_raw = inst_freq
         self.inst_freq = inst_freq - inst_freq[tidx]
         self.spectrogram = spectrogram
         self.stft_freq = freq
+        self.stft_times = times
 
         # subtract the w*t line (drive frequency line) from phase
         start = int(0.3 * tidx)
@@ -829,42 +777,40 @@ class Pixel:
             generates a new plot (True) or plots on existing plot figure (False)
         
         raw : bool, optional
-            
-        
         """
 
         if newplot:
-            fig, a = plt.subplots(nrows=3, figsize=(6,9), facecolor='white')
+            fig, a = plt.subplots(nrows=3, figsize=(6, 9), facecolor='white')
 
-        dt = 1/self.sampling_rate
+        dt = 1 / self.sampling_rate
         ridx = int(self.roi * self.sampling_rate)
         fidx = int(self.tidx)
 
         cut = self.amplitude[fidx:(fidx + ridx)]
-        
+
         cut = [fidx, (fidx + ridx)]
-        tx = np.arange(cut[0], cut[1])*dt
-        
-        a[0].plot(tx*1e3, self.inst_freq[cut[0]:cut[1]], 'r-')
-        
+        tx = np.arange(cut[0], cut[1]) * dt
+
+        a[0].plot(tx * 1e3, self.inst_freq[cut[0]:cut[1]], 'r-')
+
         if self.fit_form == 'ringdown':
-            a[1].plot(tx*1e3, self.best_fit, 'g--')
+            a[1].plot(tx * 1e3, self.best_fit, 'g--')
         else:
-            a[0].plot(tx*1e3, self.best_fit, 'g--')
-        a[1].plot(tx*1e3, self.amplitude[cut[0]:cut[1]], 'b')
-        a[2].plot(tx*1e3, self.phase[cut[0]:cut[1]]*180/np.pi, 'm') 
-        
+            a[0].plot(tx * 1e3, self.best_fit, 'g--')
+        a[1].plot(tx * 1e3, self.amplitude[cut[0]:cut[1]], 'b')
+        a[2].plot(tx * 1e3, self.phase[cut[0]:cut[1]] * 180 / np.pi, 'm')
+
         a[0].set_title('Instantaneous Frequency')
         a[0].set_ylabel('Frequency Shift (Hz)')
         a[1].set_ylabel('Amplitude (nm)')
         a[2].set_ylabel('Phase (deg)')
-        a[2].set_xlabel('Time (ms)')        
+        a[2].set_xlabel('Time (ms)')
 
         plt.tight_layout()
 
         return
 
-    def generate_inst_freq(self, timing = False):
+    def generate_inst_freq(self, timing=False):
         """
         Generates the instantaneous frequency
         
@@ -877,6 +823,9 @@ class Pixel:
             Instantaneous frequency of the signal.
         """
 
+        if timing:
+            t1 = time.time()
+
         # Remove DC component, first.
         # self.remove_dc()
 
@@ -897,26 +846,12 @@ class Pixel:
         # DWT Denoise
         # self.dwt_denoise()
 
-        if timing:
-            t1 = time.time()
-
-        if self.method == 'emd':
-
-            # Get the analytical signal doing a Hilbert transform.
-            self.hilbert_transform()
-
-            # Calculate the phase from analytic signal.
-            self.calculate_phase()
-
-            # Calculate the instantaneous frequency
-            self.EMD_inst_freq()
-
-        elif self.method == 'wavelet':
+        if self.method == 'wavelet':
 
             # Calculate instantenous frequency using wavelet transform.
             self.calculate_cwt(**self.wavelet_params)
 
-        elif self.method == 'fft':
+        elif self.method == 'stft':
 
             # Calculate instantenous frequency using sliding FFT
             self.calculate_stft(**self.fft_params)
@@ -938,29 +873,23 @@ class Pixel:
                 self.iir_filter()
 
             # Get the analytical signal doing a Hilbert transform.
-            self.hilbert_transform()
+            self.hilbert()
 
-            # Calculate the amplitude and phase from the analytic signal
-            self.calculate_amplitude()
-            
+            # Filter out oscillatory noise from amplitude
             if self.filter_amplitude:
                 
                 self.amplitude_filter()
-            
-            self.calculate_phase()
 
-            # Calculate instantenous frequency.
-            self.calculate_inst_freq()
-            
         else:
-            raise ValueError('Invalid analysis method! Valid options: hilbert, wavelet, fft, emd')
-        
+            raise ValueError('Invalid analysis method! Valid options: hilbert, wavelet, fft')
+
         if timing:
             print('Time:', time.time() - t1, 's')
-        
-        # If it's a recombination image invert it to find minimum.
-        if self.recombination:
-            self.inst_freq = self.inst_freq * -1
+
+        # Filter out oscillatory noise from instantaneous frequency
+        if self.filter_frequency:
+            
+            self.frequency_filter()
 
         return self.inst_freq, self.amplitude, self.phase
 
@@ -977,52 +906,28 @@ class Pixel:
         inst_freq : (n_points,) array_like
             Instantenous frequency of the signal.
         """
-        
-        try:
 
-            self.inst_freq, self.amplitude, self.phase = self.generate_inst_freq()
+        self.inst_freq, self.amplitude, self.phase = self.generate_inst_freq()
 
-            # Find where the minimum is.
-            if self.fit:
-
-                if self.phase_fitting:
-
-                    self.fit_phase()
-
-                else:
-
-                    if self.fit_form == 'product':
-
-                        self.fit_freq_product()
-
-                    elif self.fit_form == 'sum':
-
-                        self.fit_freq_sum()
-
-                    elif self.fit_form == 'exp':
-
-                        self.fit_freq_exp()
-                        
-                    elif self.fit_form == 'ringdown':
-
-                        self.fit_ringdown()
-            else:
-
-                self.find_minimum()
-
-            # Restore the length.
+        # If it's a recombination image invert it to find minimum.
+        if self.recombination:
             
-            if self.method == 'hilbert':
+            self.inst_freq = self.inst_freq * -1
+
+        # Find where the minimum is.
+        self.find_tfp()
+
+        # Restore the length due to FIR filter being causal
+        if self.method == 'hilbert':
             
-                    self.restore_signal()
+            self.restore_signal()
 
-        # If caught any exception, set everything to zero and log it.
-        except Exception as exception:
-            self.tfp = 0
-            self.shift = 0
-            self.inst_freq = np.zeros(self._n_points_orig)
-
-            logging.exception(exception, exc_info=True)
+        # If it's a recombination image invert it to find minimum.
+        if self.recombination:
+            
+            self.inst_freq = self.inst_freq * -1
+            self.best_fit = self.best_fit * -1
+            self.cut = self.cut * -1
 
         if self.phase_fitting:
 
@@ -1030,7 +935,4 @@ class Pixel:
 
         else:
 
-            if self.tfp == 5e-7:
-                return np.NaN, np.NaN, self.inst_freq
-
             return self.tfp, self.shift, self.inst_freq
